<?php
/**
* @package JoomShopping for Joomla!
* @subpackage payment
* @author vtb
*/
(defined('_JEXEC') || PHP_SAPI == 'cli') or die('Restricted access');

// Checking PHP Version
if (version_compare(PHP_VERSION, '7.4.0', '<')) {
    die('Payment module "Payment system VTB" requires PHP version 7.4.0 or higher.');
}

require_once __DIR__ . '/vendor/autoload.php';

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;

class pm_vtbpay extends PaymentRoot
{
    const PLUGIN_VERSION = '1.3.5';

    /**
     * @var VtbPayLogger Объект для логирования.
     */
    private VtbPayLogger $logger;


    /**
     * The constructor initializes the payment root, language file, logger and stages.
     */
    public function __construct()
    {
        JSFactory::loadExtLanguageFile('pm_vtbpay');
        JLog::addLogger(
            ['text_file' => 'com_jshopping.pm_vtbpay-' . date('d-m-Y') . '.php'],
            JLog::ALL,
            ['pm_vtbpay']
        );
    }


    /**
     * Displays the payment form for the VTB Pay payment method.
     *
     * @param array $params The parameters for the payment form.
     * @param array $pmconfigs The payment method configurations.
     */
    public function showPaymentForm($params, $pmconfigs)
    {
        include dirname(__FILE__) . '/paymentform.php';
    }


    /**
     * Displays the admin form parameters for the VTB Pay payment method plugin.
     *
     * @param array $params The parameters for the admin form.
     */
    public function showAdminFormParams($params)
    {
        $path_current_payment = str_replace(JPATH_ROOT, '', __DIR__);
        $document = JFactory::getDocument();
        $document->addScript($path_current_payment . '/assets/js/admin_vtbpay.js');

        $measure = $this->getSettingsOptions('measure');
        $taxes = $this->getSettingsOptions('taxes');
        $payments = $this->getSettingsOptions('payments');
        $subjects = $this->getSettingsOptions('subjects');

        $module_params = [
            // General
            'vtbpay_client_id',
            'vtbpay_client_secret',
            'vtbpay_merchant_authorization',
            'vtbpay_test_mode',
            'vtbpay_logging',
            'vtbpay_two_stage',
            'transaction_end_status',
            'transaction_refund_status',
            // Fiscalization
            'fiscal_enable',
            'fiscal_email',
            'fiscal_delivery',
            'fiscal_measure',
            'fiscal_tax',
            'fiscal_payment',
            'fiscal_subject'
        ];

        foreach ($module_params as $module_param) {
            if (!isset($params[$module_param])) $params[$module_param] = '';
        }

        $orders = JSFactory::getModel('orders', 'JshoppingModel');

        $callback_url = 'index.php?option=com_jshopping&controller=checkout&task=step7&js_paymentclass=' . __CLASS__ . '&act=notify&request_type=hook&no_lang=1';

        include dirname(__FILE__) . '/adminparamsform.php';
    }


    /**
     * Плучение элементов выпадающих списков для настроек.
     *
     * @param string $name Название настройки.
     * @return array
     */
    private function getSettingsOptions($name)
    {
        $options = [
            'measure' => [
                0 => 'Применяется для предметов расчета, которые могут быть реализованы поштучно или единицами',
                10 => 'Грамм',
                11 => 'Килограмм',
                12 => 'Тонна',
                20 => 'Сантиметр',
                21 => 'Дециметр',
                22 => 'Метр',
                30 => 'Квадратный сантиметр',
                31 => 'Квадратный дециметр',
                32 => 'Квадратный метр',
                40 => 'Миллилитр',
                41 => 'Литр',
                42 => 'Кубический метр',
                50 => 'Киловатт час',
                51 => 'Гигакалория',
                70 => 'Сутки (день)',
                71 => 'Час',
                72 => 'Минута',
                73 => 'Секунда',
                80 => 'Килобайт',
                81 => 'Мегабайт',
                82 => 'Гигабайт',
                83 => 'Терабайт',
                255 => 'Применяется при использовании иных единиц измерения',
            ],
            'taxes' => [
                'none' => 'Без НДС',
                'vat0' => 'НДС по ставке 0%',
                'vat10' => 'НДС чека по ставке 10%',
                'vat110' => 'НДС чека по расчетной ставке 10/110',
                'vat20' => 'НДС чека по ставке 20%',
                'vat120' => 'НДС чека по расчетной ставке 20/120',
            ],
            'payments' => [
                'full_prepayment' => 'Предоплата 100%',
                'prepayment' => 'Предоплата',
                'advance' => 'Аванс',
                'full_payment' => 'Полный расчет',
                'partial_payment' => 'Частичный расчет и кредит',
                'credit' => 'Передача в кредит',
                'credit_payment' => 'Оплата кредита'
            ],
            'subjects' => [
                1 => 'Товар',
                2 => 'Подакцизный товар',
                3 => 'Работа',
                4 => 'Услуга',
                5 => 'Ставка азартной игры',
                6 => 'Выигрыш азартной игры',
                7 => 'Ставка лотереи',
                8 => 'Выигрыш лотереи',
                9 => 'Предоставление прав',
                10 => 'Платеж',
                11 => 'Агентское вознаграждение',
                12 => 'Выплата',
                13 => 'Иной предмет расчёта',
                14 => 'Имущественное право',
                15 => 'Внереализационный доход',
                16 => 'Страховые взносы',
                17 => 'Торговый сбор',
                18 => 'Курортный сбор',
                19 => 'Залог',
                20 => 'Расход',
                21 => 'Взносы на ОПС ИП',
                22 => 'Взносы ОПС',
                23 => 'Взносы на ОМС ИП',
                24 => 'Взносы на ОМС',
                25 => 'Взносы на ОСС',
                26 => 'Платеж казино',
                27 => 'Выдача денежных средств банковским платежным агентом',
                30 => 'Подакцизный товар, подлежащий маркировке средством идентификации, не имеющий кода маркировки',
                31 => 'Подакцизный товар, подлежащий маркировке средством идентификации, имеющий код маркировки',
                32 => 'Товар, подлежащий маркировке средством идентификации, не имеющий кода маркировки, за исключением подакцизного товара',
                33 => 'Товар, подлежащий маркировке средством идентификации, имеющий код маркировки, за исключением подакцизного товара',
            ]
        ];

        return $options[$name] ?? [];
    }


    /**
     * Creates a new VtbApi instance with the given payment method configurations.
     *
     * @param array $pm_config Массив настроек модуля оплаты.
     * @return VtbApi The new VtbApi instance.
     */
    private function getVtbApi($pm_config)
    {
        return new VtbApi(
            $pm_config["vtbpay_client_id"],
            $pm_config["vtbpay_client_secret"],
            (bool) $pm_config["vtbpay_test_mode"],
            $pm_config["vtbpay_merchant_authorization"] ?? ''
        );
    }


    /**
     * Получение URL для оплаты заказа.
     *
     * @param array $pm_config Массив настроек модуля оплаты.
     * @param object $order Объект заказа.
     * @return string URL для оплаты заказа.
     */
    private function getPayUrl($pm_config, $order)
    {
        $return_url = JUri::base() .
                      'index.php?option=com_jshopping&controller=checkout&task=step7&js_paymentclass=' .
                      __CLASS__ . '&act=finish&request_type=return';

        $email = $this->getCustomerEmail($pm_config, $order);

        $two_stage = (bool) ($pm_config['vtbpay_two_stage'] ?? false);
        $fiscal_enable = (bool) ($pm_config['fiscal_enable'] ?? false);

        $items = [];
        if (!$two_stage && $fiscal_enable) $items = $this->getItems($pm_config, $order);

        $appended_order_name = 'CMS: Joomla! JoomShopping. Версия плагина: ' . self::PLUGIN_VERSION;

        // Логирование входных данных
        $this->logger->debug(
            __FUNCTION__ . ' > getOrderLink - INPUT: ', [
            'arguments' => [
                'order_id' => $order->order_id,
                'email' => $email,
                'time' => time(),
                'order_total' => $order->order_total,
                'return_url' => $return_url,
                'two_stage' => $two_stage,
                'items' => $items,
                'appended_order_name' => $appended_order_name
            ]
        ]);

        $response = $this->getVtbApi($pm_config)->getOrderLink(
            $order->order_id,
            $email,
            time(),
            $order->order_total,
            $return_url,
            $two_stage,
            $items,
            $appended_order_name
        );

        $this->logger->debug(
            __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['payUrl'] ?? '';
    }


    /**
     * Получает email клиента иначе выкидывает ошибку
     *
     * @param array $pm_config The payment method configurations.
     * @param object $order The order object.
     */
    private function getCustomerEmail($pm_config, $order)
    {
        $email = $order->email ?: $pm_config['fiscal_email'] ?? '';
        if (empty($email)) {
            throw new \Exception('Не указан E-mail.');
        }
        return $email;
    }


    /**
     * Перенаправление на форму оплаты
     *
     * @param array $pm_config The payment method configurations.
     * @param object $order The order object.
     */
    public function showEndForm($pm_config, $order)
    {
        try {
            $this->setVtbpayLogger($pm_config);
            $this->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));

            return JFactory::getApplication()->redirect($this->getPayUrl(
                $pm_config,
                $order
            ));

        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            $msg = $e->getMessage() . ' Order ID: ' . $order->order_id;
            // Handle exception and log error
            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay Exception: %s', $msg
            ), $context);

            JFactory::getApplication()->enqueueMessage($msg, 'error');
        }
    }


    /**
     * Метод проверки валидности платежа, вызывается при создании платежа, после возврата пользователя на страницу
     * подтверждения заказа, так же вызывается при приходе нотификации от платёжной системы.
     * Выполняет "проверку" транзакций (может проводить обработку запросов на Success Url, Fail Url и Result Url).
     *
     * @param $pm_config Array  Массив настроек модуля оплаты
     * @param $order     Object Объект текущего заказа, по которому происходит оформление
     * @param $act       String Тип запроса (notify, return, cancel) Возвращаем массив с результатом проверки транзакции
     *
     * @return array($rescode, $restext, $transaction, $transaction_data)
     */
    public function checkTransaction($pm_config, $order, $act)
    {
        if ($order->order_created === 0) $order->order_created = 1;
        $transactions_payment = [];
        $payment_id = '';

        try {
            $payment_status_data = $this->getPaymentStatusData($pm_config, $order->order_id);

            $transactions_payment = $payment_status_data['object']['transactions']['payments'] ?? [];
            $payment_id = end($transactions_payment)[0]['object']['paymentId'] ?? '';

            $payment_status = $payment_status_data['object']['status']['value'];

            switch ($payment_status) {
                case 'PAID':
                    $rescode = 1;
                    $order_status = (int) $pm_config['transaction_end_status'];
                    $restext = _JSHOP_CFG_VTBPAY_MSG_SUCCESSFULLY_PAID;
                    break;
                case 'PENDING':
                    $rescode = 0;
                    $order_status = (int) $pm_config['transaction_pending_status'];
                    $restext = _JSHOP_CFG_VTBPAY_MSG_SUCCESSFULLY_PAID;
                    $cart = JSFactory::getModel('cart', 'jshop');
                    $cart->clear();
                    break;
                case 'REFUNDED':
                    $rescode = 0;
                    $order_status = (int) $pm_config['transaction_refund_status'];
                    $restext = _JSHOP_CFG_VTBPAY_MSG_SUCCESSFULLY_REFUNDED;
                    break;
                default:
                    $rescode = 0;
                    $order_status = (int) $pm_config['transaction_cancel_status'];
                    $restext = _JSHOP_CFG_VTBPAY_MSG_SUCCESSFULLY_CANCELED;
                    break;
            }
        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                $e->getMessage(),
                $order->order_id ?: ''
            ), $context);

            $rescode = 0;
            $order_status = (int) $pm_config['transaction_cancel_status'];
            $restext = $e->getMessage();
        }

        $this->logger->debug(
            __FUNCTION__ . ' > change_status - INPUT: ', [
            'arguments' => [
                'order_status' => $order_status,
                'rescode' => $rescode,
                'restext' => $restext
            ]
        ]);

        $order->order_status = $order_status;
        $order->store();

        return [
            $rescode,
            $restext,
            $payment_id,
            $transactions_payment
        ];
    }


    /**
     * Вызывается при входящем вебхуке перед методом CheckTransaction().
     * Инициализирует массив с параметрами обработки входящего запроса
     *
     * @param array $pm_config Массив настроек способа оплаты
     * @param int $order_id Идентификатор заказа
     * @return array Массив с параметрами обработки входящего запроса
     */
    private function getPaymentStatusData($pm_config, $order_id)
    {
        // Logging input
        $this->logger->debug(
            __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
            'arguments' => [
                'order_id' => $order_id
            ]
        ]);

        $response = $this->getVtbApi($pm_config)->getOrderInfo(
            $order_id
        );

        $this->logger->debug(
            __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
            'response' => $response
        ]);

        return $response ?: [];
    }


    /**
     * Вызывается при входящем вебхуке перед методом CheckTransaction().
     * Инициализирует массив с параметрами обработки входящего запроса
     *
     * @param $pm_config Array Массив настроек способа оплаты
     * @return array Массив с параметрами обработки входящего запроса
     */
    public function getUrlParams($pm_config)
    {
        $params = [];
        $params['hash'] = '';
        $params['checkHash'] = 0;
        $params['checkReturnParams'] = 1;

        $this->setVtbpayLogger($pm_config);
        $input = JFactory::getApplication()->input;
        $mode = $input->getString('request_type', null);

        // Logging request data
        if (in_array($mode, ['return', 'hook'])) {
            $this->logger->setOption('additionalCommonText', $mode . '-' . rand(1111, 9999));
            $request_data = ($mode === 'return') ? $_REQUEST : json_decode(file_get_contents('php://input'), true);
            $this->logger->debug(
                __FUNCTION__ . ' > ' . $mode . ': ',
                ['request_data' => $request_data]
            );

            $params['order_id'] = $input->getString('orderId') ?: $request_data['orderId'] ?? $request_data['object']['orderId'] ?? '';
        }

        return $params;
    }


    /**
     * Получает массив товаров заказа.
     *
     * @return array Массив товаров.
     */
    private function getItems($pm_config, $order)
    {
        $get_all_items = $this->getOrderItems($order->order_id);
        $items = [];
        foreach ($get_all_items as $key => $item) {
            $product_price = number_format($item['product_item_price'], 2, '.', ',');

            if (!$item['manufacturer_code'] || $item['manufacturer_code'] == '0') {
                $code = (string) $item['product_id'];
            }
            else {
                $code = (string) $item['manufacturer_code'];
            }

            $items[] = [
                'positionId' => ($key + 1),
                'name' => $item['product_name'],
                'code' => $code,
                'price' => floatval($product_price),
                'measure' => (int) $pm_config['fiscal_measure'] ?: 0,
                'quantity' => (int) $item['product_quantity'],
                'taxParams' => [
                    'taxType' => $pm_config['fiscal_tax'] ?: 'none'
                ],
                'paymentType' => $pm_config['fiscal_payment'] ?: 'full_prepayment',
                'paymentSubject' => (int) $pm_config['fiscal_subject'] ?: 1,
                'amount' => floatval($product_price * $item['product_quantity'])
            ];
        }

        // Стоимость доставки
        if ($order->order_shipping > 0) {
            $delivery_price = floatval(number_format($order->order_shipping, 2, '.', ','));
            $items[] = [
                'positionId' => ($key + 2),
                'name' => 'Доставка',
                'price' => $delivery_price,
                'measure' => 0,
                'quantity' => 1,
                'taxParams' => [
                    'taxType' => $pm_config['fiscal_tax'] ?: 'none'
                ],
                'paymentType' => $pm_config['fiscal_delivery'] ?: 'full_prepayment',
                'paymentSubject' => 4,
                'amount' => $delivery_price
            ];
        }

        return $items;
    }


    /**
     * Получает товары заказа из бд по его идентификатору
     *
     * @return array Массив товаров заказа.
     */
    private function getOrderItems($order_id)
    {
        $db =& JFactory::getDBO();
        $q = "SELECT * FROM `#__jshopping_order_item` where `order_id`=" . $order_id;
        $db->setQuery($q);
        return $db->loadAssocList();
    }


    /**
     * Устанавливает объект логирования для модуля.
     */
    private function setVtbpayLogger($pm_config)
    {
        $logging = (bool) $pm_config['vtbpay_logging'];

        $this->logger = VtbPayLogger::getInstance()
            ->setOption('showCurrentDate', false)
            ->setOption('showLogLevel', false)
            ->setCustomRecording(function($message)
            {
                // Логирование ошибки
                JLog::add($message, JLog::ERROR, 'pm_vtbpay');

            }, VtbPayLogger::LOG_LEVEL_ERROR)
            ->setCustomRecording(function($message) use ($logging)
            {
                // Логирование сообщения дебага
                if ($logging) JLog::add($message, JLog::DEBUG, 'pm_vtbpay');

            }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}
